@@ -42,10 +42,7 @@
 #define UPDATE_TIMEOUT 50
 
 
-G_DEFINE_TYPE (GooPlayerInfo, goo_player_info, GTK_TYPE_BOX)
-
-
-struct _GooPlayerInfoPrivateData {
+struct _GooPlayerInfoPrivate {
 	GooWindow   *window;
 	GtkWidget   *cover_frame;
 	GtkWidget   *title1_label;
@@ -54,6 +51,7 @@ struct _GooPlayerInfoPrivateData {
 	GtkWidget   *time_label;
 	GtkWidget   *cover_image;
 	GtkWidget   *cover_button;
+	GtkWidget   *status_button;
 	GtkWidget   *status_image;
 	GtkWidget   *notebook;
 	char        *total_time;
@@ -64,12 +62,18 @@ struct _GooPlayerInfoPrivateData {
 	double       fraction;
 	guint        update_progress_timeout;
 	GdkPixbuf   *original_cover;
+	char        *cover_file;
 };
 
 
+G_DEFINE_TYPE_WITH_CODE (GooPlayerInfo, goo_player_info, GTK_TYPE_BOX,
+			 G_ADD_PRIVATE (GooPlayerInfo))
+
+
 enum {
 	COVER_CLICKED,
-        LAST_SIGNAL
+	UPDATE_STATUS,
+	LAST_SIGNAL
 };
 
 enum {
@@ -157,6 +161,14 @@ cover_button_clicked_cb (GtkWidget     *button,
 }
 
 
+static void
+status_button_clicked_cb (GtkWidget     *button,
+			  GooPlayerInfo *info)
+{
+	g_signal_emit (info, goo_player_info_signals[UPDATE_STATUS], 0);
+}
+
+
 /* -- drag and drop -- */
 
 
@@ -195,8 +207,8 @@ cover_button_drag_data_received  (GtkWidget          *widget,
 static void
 goo_player_info_construct (GooPlayerInfo *info)
 {
-	GooPlayerInfoPrivateData *priv;
-	GtkWidget *vbox;
+	GooPlayerInfoPrivate *priv;
+	GtkWidget            *vbox;
 
 	priv = info->priv;
 
@@ -205,9 +217,9 @@ goo_player_info_construct (GooPlayerInfo *info)
 	priv->update_id = 0;
 
 	gtk_widget_set_margin_top (GTK_WIDGET (info), 10);
-	gtk_widget_set_margin_right (GTK_WIDGET (info), 10);
+	gtk_widget_set_margin_end (GTK_WIDGET (info), 10);
 	gtk_widget_set_margin_bottom (GTK_WIDGET (info), 10);
-	gtk_widget_set_margin_left (GTK_WIDGET (info), 10);
+	gtk_widget_set_margin_start (GTK_WIDGET (info), 10);
 
 	gtk_widget_set_can_focus (GTK_WIDGET (info), FALSE);
 	gtk_box_set_spacing (GTK_BOX (info), 12);
@@ -217,21 +229,25 @@ goo_player_info_construct (GooPlayerInfo *info)
 
 	priv->title1_label = gtk_label_new (NULL);
 	gtk_style_context_add_class (gtk_widget_get_style_context (priv->title1_label), "goobox-info-album");
-	gtk_misc_set_alignment (GTK_MISC (priv->title1_label), 0.0, 0.5);
+	gtk_label_set_xalign (GTK_LABEL (priv->title1_label), 0.0);
+	gtk_label_set_yalign (GTK_LABEL (priv->title1_label), 0.5);
 
 	priv->title2_label = gtk_label_new (NULL);
 	gtk_style_context_add_class (gtk_widget_get_style_context (priv->title2_label), "goobox-info-artist");
-	gtk_misc_set_alignment (GTK_MISC (priv->title2_label), 0.0, 0.5);
+	gtk_label_set_xalign (GTK_LABEL (priv->title2_label), 0.0);
+	gtk_label_set_yalign (GTK_LABEL (priv->title2_label), 0.5);
 	gtk_label_set_selectable (GTK_LABEL (priv->title2_label), TRUE);
 
 	priv->title3_label = gtk_label_new (NULL);
 	gtk_style_context_add_class (gtk_widget_get_style_context (priv->title3_label), "goobox-info-track");
-	gtk_misc_set_alignment (GTK_MISC (priv->title3_label), 0.0, 0.5);
+	gtk_label_set_xalign (GTK_LABEL (priv->title3_label), 0.0);
+	gtk_label_set_yalign (GTK_LABEL (priv->title3_label), 0.5);
 	gtk_label_set_selectable (GTK_LABEL (priv->title3_label), TRUE);
 
 	priv->time_label = gtk_label_new (NULL);
 	gtk_style_context_add_class (gtk_widget_get_style_context (priv->time_label), "goobox-info-time");
-	gtk_misc_set_alignment (GTK_MISC (priv->time_label), 0.0, 0.5);
+	gtk_label_set_xalign (GTK_LABEL (priv->time_label), 0.0);
+	gtk_label_set_yalign (GTK_LABEL (priv->time_label), 0.5);
 	gtk_label_set_selectable (GTK_LABEL (priv->time_label), TRUE);
 
 	gtk_label_set_ellipsize (GTK_LABEL (priv->title1_label), PANGO_ELLIPSIZE_END);
@@ -273,9 +289,17 @@ goo_player_info_construct (GooPlayerInfo *info)
 
 	/* Status image */
 
+	priv->status_button = gtk_button_new ();
+	gtk_button_set_relief (GTK_BUTTON (priv->status_button), GTK_RELIEF_NONE);
+	g_signal_connect (G_OBJECT (priv->status_button),
+			  "clicked",
+			  G_CALLBACK (status_button_clicked_cb),
+			  info);
+
 	priv->status_image = gtk_image_new_from_icon_name (GOO_ICON_NAME_NO_DISC, GTK_ICON_SIZE_DIALOG);
 	gtk_widget_set_size_request (priv->status_image, COVER_SIZE, COVER_SIZE);
 	gtk_widget_show (priv->cover_image);
+	gtk_container_add (GTK_CONTAINER (priv->status_button), priv->status_image);
 	/*gtk_container_set_border_width (GTK_CONTAINER (priv->status_image), 6);*/
 
 	/* Frame */
@@ -286,7 +310,7 @@ goo_player_info_construct (GooPlayerInfo *info)
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (priv->notebook), FALSE);
 	gtk_widget_show (priv->notebook);
 
-	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), priv->status_image, NULL);
+	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), priv->status_button, NULL);
 	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), priv->cover_button, NULL);
 
 	priv->cover_frame = gtk_frame_new (NULL);
@@ -322,6 +346,7 @@ goo_player_info_finalize (GObject *object)
 	info = GOO_PLAYER_INFO (object);
 	g_free (info->priv->total_time);
 	if (info->priv->update_id != 0) {
+		g_free (info->priv->cover_file);
 		g_object_unref (info->priv->original_cover);
 		g_source_remove (info->priv->update_id);
 		info->priv->update_id = 0;
@@ -369,10 +394,10 @@ show_all_labels (GooPlayerInfo *info)
 static void
 goo_player_info_update_state (GooPlayerInfo *info)
 {
-	GooPlayerInfoPrivateData *priv = info->priv;
-	GooPlayerState  state;
-	AlbumInfo      *album;
-	GooPlayer      *player;
+	GooPlayerInfoPrivate *priv = info->priv;
+	GooPlayerState        state;
+	AlbumInfo            *album;
+	GooPlayer            *player;
 
 	if (info->priv->window == NULL)
 		return;
@@ -470,10 +495,8 @@ static void
 goo_player_info_set_total_time (GooPlayerInfo  *info,
 				gint64          total_time)
 {
-	GooPlayerInfoPrivateData *priv = info->priv;
-
-	g_free (priv->total_time);
-	priv->total_time = (total_time > 0) ? _g_format_duration_for_display (total_time * 1000) : NULL;
+	g_free (info->priv->total_time);
+	info->priv->total_time = (total_time > 0) ? _g_format_duration_for_display (total_time * 1000) : NULL;
 	goo_player_info_update_state (info);
 }
 
@@ -510,6 +533,10 @@ goo_player_info_set_cover (GooPlayerInfo *info,
 		return;
 
 	g_clear_object (&info->priv->original_cover);
+	if (info->priv->cover_file != NULL) {
+		g_free (info->priv->cover_file);
+		info->priv->cover_file = NULL;
+	}
 
 	if (strcmp (cover, "no-disc") == 0) {
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (info->priv->notebook), 0);
@@ -530,6 +557,7 @@ goo_player_info_set_cover (GooPlayerInfo *info,
 					      GTK_ICON_SIZE_DIALOG);
 	}
 	else {
+		info->priv->cover_file = g_strdup (cover);
 		info->priv->original_cover = gdk_pixbuf_new_from_file (cover, NULL);
 		if (info->priv->original_cover != NULL) {
 			GdkPixbuf *image;
@@ -582,8 +610,6 @@ goo_player_info_class_init (GooPlayerInfoClass *class)
         GObjectClass   *gobject_class;
 	GtkWidgetClass *widget_class;
 
-	g_type_class_add_private (class, sizeof (GooPlayerInfoPrivateData));
-
 	gobject_class = G_OBJECT_CLASS (class);
         gobject_class->finalize = goo_player_info_finalize;
 
@@ -591,7 +617,7 @@ goo_player_info_class_init (GooPlayerInfoClass *class)
 	widget_class->get_preferred_width = goo_player_info_get_preferred_width;
 
 	goo_player_info_signals[COVER_CLICKED] =
-                g_signal_new ("cover_clicked",
+		g_signal_new ("cover_clicked",
 			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GooPlayerInfoClass, cover_clicked),
@@ -599,6 +625,15 @@ goo_player_info_class_init (GooPlayerInfoClass *class)
 			      goo_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+	goo_player_info_signals[UPDATE_STATUS] =
+		g_signal_new ("update-status",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GooPlayerInfoClass, update_status),
+			      NULL, NULL,
+			      goo_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
 }
 
 
@@ -606,7 +641,7 @@ static void
 goo_player_info_init (GooPlayerInfo *info)
 {
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (info), GTK_ORIENTATION_HORIZONTAL);
-	info->priv = G_TYPE_INSTANCE_GET_PRIVATE (info, GOO_TYPE_PLAYER_INFO, GooPlayerInfoPrivateData);
+	info->priv = goo_player_info_get_instance_private (info);
 }
 
 
@@ -652,3 +687,10 @@ goo_player_info_get_cover (GooPlayerInfo *info)
 {
 	return info->priv->original_cover;
 }
+
+
+const char *
+goo_player_info_get_cover_file (GooPlayerInfo *info)
+{
+	return info->priv->cover_file;
+}
