@@ -30,8 +30,8 @@
 #include "dlg-cover-chooser.h"
 #include "goo-marshal.h"
 #include "goo-player.h"
-#include "goo-player-bar.h"
 #include "goo-player-info.h"
+#include "goo-player-progress.h"
 #include "goo-window.h"
 #include "goo-window-actions-entries.h"
 #include "gth-user-dir.h"
@@ -41,7 +41,6 @@
 #include "main.h"
 #include "preferences.h"
 #include "typedefs.h"
-#include "icons/pixbufs.h"
 
 #define FILES_TO_PROCESS_AT_ONCE 500
 #define HIDE_TRACK_LIST N_("Hide _tracks")
@@ -73,7 +72,8 @@ struct _GooWindowPrivate {
 	GtkWidget         *message_label;
 	GtkWidget         *message_bar_properties_button;
 	GtkWidget         *info;
-	GtkWidget         *player_bar;
+	GtkWidget         *play_button;
+	GtkWidget         *progress;
 
 	guint              first_time_event;
 	guint              next_timeout_handle;
@@ -127,7 +127,8 @@ enum {
 };
 
 
-G_DEFINE_TYPE (GooWindow, goo_window, GTK_TYPE_APPLICATION_WINDOW)
+G_DEFINE_TYPE_WITH_CODE (GooWindow, goo_window, GTK_TYPE_APPLICATION_WINDOW,
+			 G_ADD_PRIVATE (GooWindow))
 
 
 static void
@@ -400,7 +401,7 @@ goo_window_finalize (GObject *object)
 
 		/* Save preferences */
 
-		g_settings_set_int (window->priv->settings_general, PREF_GENERAL_VOLUME, (int) (goo_player_get_audio_volume (window->priv->player) * 100.0));
+		g_settings_set_int (window->priv->settings_general, PREF_GENERAL_VOLUME, goo_player_get_audio_volume (window->priv->player));
 
 		/**/
 
@@ -871,11 +872,11 @@ pref_playlist_shuffle_changed (GSettings  *settings,
 static void
 save_window_size (GooWindow *window)
 {
-	GdkWindow *w;
+	int width, height;
 
-	w = gtk_widget_get_window (GTK_WIDGET (window));
-	g_settings_set_int (window->priv->settings_ui, PREF_UI_WINDOW_WIDTH, gdk_window_get_width (w));
-	g_settings_set_int (window->priv->settings_ui, PREF_UI_WINDOW_HEIGHT, gdk_window_get_height (w));
+	gtk_window_get_size (GTK_WINDOW (window), &width, &height);
+	g_settings_set_int (window->priv->settings_ui, PREF_UI_WINDOW_WIDTH, width);
+	g_settings_set_int (window->priv->settings_ui, PREF_UI_WINDOW_HEIGHT, height);
 }
 
 
@@ -932,8 +933,6 @@ goo_window_class_init (GooWindowClass *class)
 	GObjectClass   *gobject_class;
 	GtkWidgetClass *widget_class;
 
-	g_type_class_add_private (class, sizeof (GooWindowPrivate));
-
 	gobject_class = (GObjectClass*) class;
 	gobject_class->finalize = goo_window_finalize;
 
@@ -1138,43 +1137,42 @@ get_action_name (GooPlayerAction action)
 static gboolean
 notify_current_state_cb (gpointer user_data)
 {
-	GooWindow *window = user_data;
+	GooWindow      *window = user_data;
+	GString        *info;
+	GooPlayerState  state;
 
 	if (window->priv->notify_event != 0) {
 		g_source_remove (window->priv->notify_event);
 		window->priv->notify_event = 0;
 	}
 
-#ifdef ENABLE_NOTIFICATION
-
-	GString        *info = g_string_new ("");
-	GooPlayerState  state;
-
 	state = goo_player_get_state (window->priv->player);
 
 	if ((state == GOO_PLAYER_STATE_ERROR) || (state == GOO_PLAYER_STATE_NO_DISC)) {
-		system_notify (window, _("No disc"), "");
+		system_notify (window, "no-disc", _("No disc"), NULL);
 		return FALSE;
 	}
 
 	if (state == GOO_PLAYER_STATE_DATA_DISC) {
-		system_notify (window, _("Data disc"), "");
+		system_notify (window, "data-disc", _("Data disc"), NULL);
 		return FALSE;
 	}
 
 	if (window->priv->album == NULL) {
-		system_notify (window, "", "");
+		system_notify (window, "no-album", "", NULL);
 		return FALSE;
 	}
 
 	if (window->priv->current_track == NULL) {
 		if ((window->priv->album != NULL) && (window->priv->album->title != NULL))
-			system_notify (window, window->priv->album->title , "");
+			system_notify (window, "new-album", window->priv->album->title , NULL);
 		else
-			system_notify (window, _("Audio CD"), "");
+			system_notify (window, "audio-cd", _("Audio CD"), NULL);
 		return FALSE;
 	}
 
+	info = g_string_new ("");
+
 	if (window->priv->album->artist != NULL) {
 		char *e_artist = g_markup_escape_text (window->priv->album->artist, -1);
 
@@ -1183,24 +1181,18 @@ notify_current_state_cb (gpointer user_data)
 	}
 
 	if (window->priv->album->title != NULL) {
-		char *e_album = g_markup_escape_text (window->priv->album->title, -1);
-
-		g_string_append (info, "\n");
-
-		g_string_append_printf (info, "<i>%s</i>", e_album);
-		g_free (e_album);
+		g_string_append (info, " - ");
+		g_string_append_printf (info, "%s", window->priv->album->title);
 	}
 
-	g_string_append (info, " ");
-
-	system_notify (window,
-		       window->priv->current_track->title,
-		       info->str);
+	if (goo_player_get_state (goo_window_get_player (window)) == GOO_PLAYER_STATE_PLAYING)
+		system_notify (window,
+			       "new-track",
+			       window->priv->current_track->title,
+			       (strlen (info->str) > 0) ? info->str : NULL);
 
 	g_string_free (info, TRUE);
 
-#endif /* ENABLE_NOTIFICATION */
-
 	return FALSE;
 }
 
@@ -1258,7 +1250,6 @@ player_start_cb (GooPlayer       *player,
 		break;
 
 	case GOO_PLAYER_ACTION_METADATA:
-		notify_current_state (window, action);
 		goo_window_show_message_bar_action (window, _("Searching disc information…"));
 		break;
 
@@ -1324,7 +1315,7 @@ goo_window_set_current_track (GooWindow *window,
 static void
 window_update_title (GooWindow *window)
 {
-	gtk_window_set_title (GTK_WINDOW (window), _("CD Player"));
+	goo_player_progress_set_title (GOO_PLAYER_PROGRESS (window->priv->progress), _("CD Player"));
 
 #if 0
 	GooPlayerState  state;
@@ -1348,7 +1339,7 @@ window_update_title (GooWindow *window)
 		break;
 	}
 
-	gtk_window_set_title (GTK_WINDOW (window), title->str);
+	goo_player_progress_set_title (GOO_PLAYER_PROGRESS (window->priv->progress), title->str);
 
 	g_string_free (title, TRUE);
 #endif
@@ -1525,18 +1516,23 @@ goo_window_update_album (GooWindow *window)
 }
 
 
+static void
+window_update_play_button_state (GooWindow *window)
+{
+	gboolean playing = (goo_player_get_state (window->priv->player) == GOO_PLAYER_STATE_PLAYING);
+	gtk_button_set_image (GTK_BUTTON (window->priv->play_button),
+			      gtk_image_new_from_icon_name (playing ? GOO_ICON_NAME_PAUSE : GOO_ICON_NAME_PLAY, GTK_ICON_SIZE_MENU));
+}
+
+
 static void
 player_done_cb (GooPlayer       *player,
 		GooPlayerAction  action,
 		GError          *error,
 		GooWindow       *window)
 {
-	gboolean rtl;
-
 	debug (DEBUG_INFO, "DONE [%s]\n", get_action_name (action));
 
-	rtl = gtk_widget_get_direction (GTK_WIDGET (window)) == GTK_TEXT_DIR_RTL;
-
 	switch (action) {
 	case GOO_PLAYER_ACTION_LIST:
 		goo_window_update_album (window);
@@ -1560,30 +1556,33 @@ player_done_cb (GooPlayer       *player,
 		}
 		else
 			goo_window_hide_message_bar (window);
+		notify_current_state (window, action);
 		auto_fetch_cover_image (window);
 		break;
 
 	case GOO_PLAYER_ACTION_SEEK_SONG:
 		goo_window_set_current_track (window, goo_player_get_current_track (window->priv->player));
 		goo_window_select_current_track (window);
-		set_current_track_icon (window, rtl ? GOO_ICON_NAME_PLAY_RTL : GOO_ICON_NAME_PLAY);
+		set_current_track_icon (window, GOO_ICON_NAME_PLAY);
 		break;
 
 	case GOO_PLAYER_ACTION_PLAY:
 	case GOO_PLAYER_ACTION_STOP:
 	case GOO_PLAYER_ACTION_MEDIUM_REMOVED:
 		if (action == GOO_PLAYER_ACTION_PLAY) {
-			set_current_track_icon (window, rtl ? GOO_ICON_NAME_PLAY_RTL : GOO_ICON_NAME_PLAY);
+			set_current_track_icon (window, GOO_ICON_NAME_PLAY);
 			window->priv->next_timeout_handle = g_idle_add (next_time_idle, window);
 		}
 		else if (action == GOO_PLAYER_ACTION_STOP)
 			set_current_track_icon (window, GOO_ICON_NAME_STOP);
 		notify_current_state (window, action);
+		window_update_play_button_state (window);
 		break;
 
 	case GOO_PLAYER_ACTION_PAUSE:
 		set_current_track_icon (window, GOO_ICON_NAME_PAUSE);
 		notify_current_state (window, action);
+		window_update_play_button_state (window);
 		break;
 
 	case GOO_PLAYER_ACTION_STARTED_NEXT:
@@ -1604,6 +1603,7 @@ player_state_changed_cb (GooPlayer *player,
 {
 	window_update_sensitivity (window);
 	window_update_title (window);
+	window_update_play_button_state (window);
 }
 
 
@@ -1682,11 +1682,7 @@ file_button_press_cb (GtkWidget      *widget,
 		else
 			gtk_tree_selection_unselect_all (selection);
 
-		gtk_menu_popup (GTK_MENU (window->priv->file_popup_menu),
-				NULL, NULL, NULL,
-				window,
-				event->button,
-				event->time);
+		gtk_menu_popup_at_pointer (GTK_MENU (window->priv->file_popup_menu), (GdkEvent *) event);
 		return TRUE;
 	}
 	else if ((event->type == GDK_BUTTON_PRESS) && (event->button == 1)) {
@@ -1772,7 +1768,6 @@ update_ui_from_expander_state (GooWindow *window)
 		gtk_window_resize (GTK_WINDOW (window),
 				   g_settings_get_int (window->priv->settings_ui, PREF_UI_WINDOW_WIDTH),
 				   g_settings_get_int (window->priv->settings_ui, PREF_UI_WINDOW_HEIGHT));
-	gtk_window_set_has_resize_grip (GTK_WINDOW (window), TRUE);
 
 	hints.max_height = window->priv->resizable_playlist ? G_MAXINT : -1;
 	hints.max_width = G_MAXINT;
@@ -1783,27 +1778,30 @@ update_ui_from_expander_state (GooWindow *window)
 }
 
 
-static void
-player_bar_skip_to_cb (GooPlayerBar *info,
-		       int           seconds,
-		       GooWindow    *window)
-{
-	debug (DEBUG_INFO, "[Window] skip to %d\n", seconds);
-	goo_player_skip_to (window->priv->player, (guint) seconds);
-}
-
-
 static void
 player_info_cover_clicked_cb (GooPlayerInfo *info,
 			      GooWindow     *window)
 {
 	debug (DEBUG_INFO, "[Window] cover clicked\n");
+	gtk_menu_popup_at_widget (GTK_MENU (window->priv->cover_popup_menu),
+				  GTK_WIDGET (info),
+				  GDK_GRAVITY_SOUTH_WEST,
+				  GDK_GRAVITY_NORTH_WEST,
+				  NULL);
+}
 
-	gtk_menu_popup (GTK_MENU (window->priv->cover_popup_menu),
-			NULL, NULL, NULL,
-			window,
-			3,
-			GDK_CURRENT_TIME);
+
+static void
+player_info_update_status_cb (GooPlayerInfo *info,
+			      GooWindow     *window)
+{
+	BraseroDrive *drive;
+
+	debug (DEBUG_INFO, "[Window] update status\n");
+
+	drive = goo_player_get_drive (window->priv->player);
+	if (drive != NULL)
+		brasero_drive_reprobe (drive);
 }
 
 
@@ -1899,13 +1897,17 @@ window_size_allocate_cb (GtkWidget    *widget,
 	                 GdkRectangle *allocation,
 	                 gpointer      user_data)
 {
-	GooWindow *window = user_data;
-	int        max_window_height;
+	GooWindow    *window = user_data;
+	GdkRectangle  geometry;
+	int           max_window_height;
 
 	if (window->priv->resizable_playlist)
 		return;
 
-	max_window_height = gdk_screen_get_height (gtk_widget_get_screen (GTK_WIDGET (window))) * MAX_WINDOW_HEIGHT_PERCENTAGE;
+	if (!_gtk_widget_get_monitor_geometry (GTK_WIDGET (window), &geometry))
+		return;
+
+	max_window_height = (float) geometry.height * MAX_WINDOW_HEIGHT_PERCENTAGE;
 	if (allocation->height > max_window_height) {
 		GdkGeometry hints;
 
@@ -1928,10 +1930,34 @@ window_size_allocate_cb (GtkWidget    *widget,
 }
 
 
+static void
+_gtk_menu_button_set_style_for_header_bar (GtkWidget *button)
+{
+	GtkStyleContext *context;
+
+	gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
+	context = gtk_widget_get_style_context (button);
+	gtk_style_context_add_class (context, "image-button");
+	gtk_style_context_remove_class (context, "text-button");
+}
+
+
+static GtkWidget *
+_gtk_image_button_new_for_header_bar (const char *icon_name)
+	{
+	GtkWidget *button;
+
+	button = gtk_button_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
+	_gtk_menu_button_set_style_for_header_bar (button);
+
+	return button;
+}
+
+
 static void
 goo_window_init (GooWindow *window)
 {
-	window->priv = G_TYPE_INSTANCE_GET_PRIVATE (window, GOO_TYPE_WINDOW, GooWindowPrivate);
+	window->priv = goo_window_get_instance_private (window);
 	window->priv->exiting = FALSE;
 	window->priv->check_id = 0;
 	window->priv->url_list = NULL;
@@ -1942,12 +1968,11 @@ goo_window_init (GooWindow *window)
 
 	gtk_window_add_accel_group (GTK_WINDOW (window), window->priv->accel_group);
 
-	gtk_window_set_title (GTK_WINDOW (window), _("CD Player"));
 
 	g_action_map_add_action_entries (G_ACTION_MAP (window),
 					 goo_window_actions,
 					 G_N_ELEMENTS (goo_window_actions),
-                                       	 window);
+					 window);
 	goo_window_add_accelerators (window,
 				     goo_window_accelerators,
 				     G_N_ELEMENTS (goo_window_accelerators));
@@ -2081,6 +2106,26 @@ message_bar_response_cb (GtkInfoBar *info_bar,
 }
 
 
+static void
+progress_skip_to_cb (GooPlayerProgress *progress,
+		     int                seconds,
+		     GooWindow         *window)
+{
+	debug (DEBUG_INFO, "[Window] skip to %d\n", seconds);
+	goo_player_skip_to (window->priv->player, (guint) seconds);
+}
+
+
+static void
+volume_button_value_changed_cb (GtkScaleButton *button,
+				gdouble         value,
+				gpointer        user_data)
+{
+	GooWindow *window = user_data;
+	goo_player_set_audio_volume (window->priv->player, value);
+}
+
+
 static void
 goo_window_construct (GooWindow    *window,
 		      BraseroDrive *drive)
@@ -2090,6 +2135,7 @@ goo_window_construct (GooWindow    *window,
 	GtkWidget        *vbox;
 	GtkWidget        *hbox;
 	GtkTreeSelection *selection;
+	GtkWidget        *headerbar;
 
 	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (window)), "goobox-main-window");
 	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (window)), GTK_STYLE_CLASS_VIEW);
@@ -2106,9 +2152,17 @@ goo_window_construct (GooWindow    *window,
 	if (icon_size == 0)
 		icon_size = get_icon_size_from_settings (GTK_ICON_SIZE_LARGE_TOOLBAR);
 
+	/* Create the settings objects */
+
+	window->priv->settings_general = g_settings_new (GOOBOX_SCHEMA_GENERAL);
+	window->priv->settings_ui = g_settings_new (GOOBOX_SCHEMA_UI);
+	window->priv->settings_playlist = g_settings_new (GOOBOX_SCHEMA_PLAYLIST);
+	window->priv->settings_encoder = g_settings_new (GOOBOX_SCHEMA_ENCODER);
+
 	/* Create the data */
 
 	window->priv->player = goo_player_new (drive);
+	goo_player_set_audio_volume (window->priv->player, g_settings_get_int (window->priv->settings_general, PREF_GENERAL_VOLUME));
 
 	g_signal_connect (window->priv->player,
 			  "start",
@@ -2125,13 +2179,6 @@ goo_window_construct (GooWindow    *window,
 
 	window->priv->playlist = NULL;
 
-	/* Create the settings objects */
-
-	window->priv->settings_general = g_settings_new (GOOBOX_SCHEMA_GENERAL);
-	window->priv->settings_ui = g_settings_new (GOOBOX_SCHEMA_UI);
-	window->priv->settings_playlist = g_settings_new (GOOBOX_SCHEMA_PLAYLIST);
-	window->priv->settings_encoder = g_settings_new (GOOBOX_SCHEMA_ENCODER);
-
 	/* Create the widgets. */
 
 	/* * File list. */
@@ -2145,7 +2192,6 @@ goo_window_construct (GooWindow    *window,
 						       G_TYPE_STRING);
 	window->priv->list_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (window->priv->list_store));
 
-	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (window->priv->list_view), TRUE);
 	add_columns (window, GTK_TREE_VIEW (window->priv->list_view));
 	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (window->priv->list_view), TRUE);
 	gtk_tree_view_set_search_column (GTK_TREE_VIEW (window->priv->list_view), COLUMN_TITLE);
@@ -2242,20 +2288,16 @@ goo_window_construct (GooWindow    *window,
 			  "cover_clicked",
 			  G_CALLBACK (player_info_cover_clicked_cb),
 			  window);
+	g_signal_connect (window->priv->info,
+			  "update-status",
+			  G_CALLBACK (player_info_update_status_cb),
+			  window);
 	gtk_box_pack_start (GTK_BOX (hbox), window->priv->info, TRUE, TRUE, 0);
 
 	/**/
 
 	gtk_box_pack_start (GTK_BOX (vbox), scrolled_window, TRUE, TRUE, 0);
 
-	window->priv->player_bar = goo_player_bar_new (window->priv->player, G_ACTION_MAP (window));
-	g_signal_connect (window->priv->player_bar,
-			  "skip-to",
-			  G_CALLBACK (player_bar_skip_to_cb),
-			  window);
-	gtk_widget_show (window->priv->player_bar);
-	gtk_box_pack_start (GTK_BOX (vbox), window->priv->player_bar, FALSE, FALSE, 0);
-
 	/**/
 
 	gtk_widget_show_all (vbox);
@@ -2271,6 +2313,105 @@ goo_window_construct (GooWindow    *window,
 				     g_settings_get_int (window->priv->settings_ui, PREF_UI_WINDOW_WIDTH),
 				     g_settings_get_int (window->priv->settings_ui, PREF_UI_WINDOW_HEIGHT));
 
+	headerbar = gtk_header_bar_new ();
+	gtk_widget_show (headerbar);
+	gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (headerbar), TRUE);
+
+	/* play buttons */
+
+	{
+		GtkWidget *button;
+		GtkWidget *button_box;
+
+		window->priv->play_button = button = _gtk_image_button_new_for_header_bar (GOO_ICON_NAME_PLAY);
+		gtk_actionable_set_action_name (GTK_ACTIONABLE (button), "win.toggle-play");
+		gtk_widget_show_all (button);
+		gtk_header_bar_pack_start (GTK_HEADER_BAR (headerbar), button);
+
+		button_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+		gtk_style_context_add_class (gtk_widget_get_style_context (button_box), GTK_STYLE_CLASS_LINKED);
+		gtk_widget_show (button_box);
+		gtk_header_bar_pack_start (GTK_HEADER_BAR (headerbar), button_box);
+
+		button = _gtk_image_button_new_for_header_bar (GOO_ICON_NAME_PREV);
+		gtk_actionable_set_action_name (GTK_ACTIONABLE (button), "win.previous-track");
+		gtk_widget_show_all (button);
+		gtk_box_pack_start (GTK_BOX (button_box), button, FALSE, FALSE, 0);
+
+		button = _gtk_image_button_new_for_header_bar (GOO_ICON_NAME_NEXT);
+		gtk_actionable_set_action_name (GTK_ACTIONABLE (button), "win.next-track");
+		gtk_widget_show_all (button);
+		gtk_box_pack_start (GTK_BOX (button_box), button, FALSE, FALSE, 0);
+
+		button = gtk_volume_button_new ();
+		gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NORMAL);
+		gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
+		gtk_scale_button_set_adjustment (GTK_SCALE_BUTTON (button),
+						 gtk_adjustment_new (0.0,
+								     0.0,
+								     100.0,
+								     1.0,
+								     10.0,
+								     0.0));
+		gtk_scale_button_set_value (GTK_SCALE_BUTTON (button), goo_player_get_audio_volume (window->priv->player));
+		gtk_widget_show (button);
+		gtk_header_bar_pack_start (GTK_HEADER_BAR (headerbar), button);
+
+		g_signal_connect (button,
+				  "value-changed",
+				  G_CALLBACK (volume_button_value_changed_cb),
+				  window);
+	}
+
+	/* gears menu button */
+
+	{
+		GtkBuilder *builder;
+		GMenuModel *menu;
+		GtkWidget  *button;
+
+		builder = _gtk_builder_new_from_resource ("gears-menu.ui");
+		menu = G_MENU_MODEL (gtk_builder_get_object (builder, "gears-menu"));
+		button = gtk_menu_button_new ();
+		_gtk_menu_button_set_style_for_header_bar (button);
+		gtk_menu_button_set_direction (GTK_MENU_BUTTON (button), GTK_ARROW_NONE);
+		gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (button), menu);
+		gtk_menu_button_set_use_popover (GTK_MENU_BUTTON (button), FALSE);
+		gtk_widget_show_all (button);
+		gtk_header_bar_pack_end (GTK_HEADER_BAR (headerbar), button);
+
+		_gtk_window_add_accelerators_from_menu ((GTK_WINDOW (window)), menu);
+
+		g_object_unref (builder);
+	}
+
+	/* extract button */
+
+	{
+		GtkWidget *button;
+
+		button = _gtk_image_button_new_for_header_bar (GOO_ICON_NAME_EXTRACT);
+		gtk_widget_set_tooltip_text (button, _("Extract Tracks"));
+		gtk_actionable_set_action_name (GTK_ACTIONABLE (button), "win.extract");
+		gtk_widget_show_all (button);
+		gtk_header_bar_pack_end (GTK_HEADER_BAR (headerbar), button);
+	}
+
+	/* custom title */
+
+	window->priv->progress = goo_player_progress_new (window->priv->player);
+	gtk_widget_show (window->priv->progress);
+	gtk_header_bar_set_custom_title (GTK_HEADER_BAR (headerbar), window->priv->progress);
+
+	g_signal_connect (window->priv->progress,
+			  "skip-to",
+			  G_CALLBACK (progress_skip_to_cb),
+			  window);
+
+	gtk_window_set_titlebar (GTK_WINDOW (window), headerbar);
+
+	goo_player_progress_set_title (GOO_PLAYER_PROGRESS (window->priv->progress), _("CD Player"));
+
 	/* Add notification callbacks. */
 
 	g_signal_connect (window->priv->settings_playlist,
@@ -2508,6 +2649,14 @@ void
 goo_window_set_drive (GooWindow    *window,
 		      BraseroDrive *drive)
 {
+	BraseroDrive *current_drive;
+
+	current_drive = goo_player_get_drive (window->priv->player);
+	if ((drive == NULL) && (current_drive == NULL))
+		return;
+	if ((drive != NULL) && (current_drive != NULL) && (g_strcmp0 (brasero_drive_get_device (drive), brasero_drive_get_device (current_drive)) == 0))
+		return;
+
 	goo_player_set_drive (window->priv->player, drive);
 	goo_player_update (window->priv->player);
 }
@@ -2786,13 +2935,18 @@ goo_window_search_cover_on_internet (GooWindow *window)
 	debug (DEBUG_INFO, "SEARCH ON INTERNET\n");
 
 	if ((window->priv->album->title == NULL) || (window->priv->album->artist == NULL)) {
-		_gtk_message_dialog_new (GTK_WINDOW (window),
-					 GTK_DIALOG_MODAL,
-					 _GTK_ICON_NAME_DIALOG_ERROR,
-				         _("Could not search for a cover on Internet"),
-				         _("You have to enter the artist and album names in order to find the album cover."),
-				         _GTK_LABEL_CLOSE, GTK_RESPONSE_CLOSE,
-				         NULL);
+		GtkWidget *d;
+
+		d = _gtk_error_dialog_new (GTK_WINDOW (window),
+					   GTK_DIALOG_MODAL,
+					   NULL,
+					   _("Could not search for a cover on Internet"),
+					   "%s",
+					   _("You have to enter the artist and album names in order to find the album cover."));
+		g_signal_connect (G_OBJECT (d), "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+		gtk_widget_show (d);
 		return;
 	}
 
